%**************************************************************************
%                   Hochschule Hamm-Lippstadt                             *
%**************************************************************************
% Modul           : colorSensor.m                                         *
%                                                                         *
% Datum           : 06. Januar 2023                                       *
%                                                                         *
% Funktion        : MATLAB System Class: Auslesen des TCS34725-I2C-Sensor *
%                                                                         *
% Implementierung : MATLAB R2022b                                         *
%                                                                         *
% Req. Toolbox    : MATLAB Coder, Simulink Support package for Arduino HW *
%                                                                         *
% Autor           : Christabelle Feunang, Christian Schwinne              *
%                                                                         *
% Bemerkung       : Basiert auf der MATLAB generierten Vorlage "Sink.m"   *
%                                                                         *
%**************************************************************************

% Die Klasse basiert auf Code aus offizieller MathWorks-Dokumentation:
% https://de.mathworks.com/help/supportpkg/arduino/ug/create-custom-blocks-for-arduino-support-package.html

classdef colorSensor < realtime.internal.SourceSampleTime & ...
        coder.ExternalDependency
    %
    % System object template for a source block.
    % 
    % This template includes most, but not all, possible properties,
    % attributes, and methods that you can implement for a System object in
    % Simulink.
    %
    % NOTE: When renaming the class name Source, the file name and
    % constructor name must be updated to use the class name.
    %
    
    % Copyright 2016-2018 The MathWorks, Inc.
    %#codegen
    %#ok<*EMCA>
    
    properties
        % Public, tunable properties.
    end
    
    properties (Nontunable)
        % Public, non-tunable properties.
    end
    
    properties (Access = private)
        % Pre-computed constants.
    end
    
    methods
        % Constructor
        function obj = colorSensor(varargin)
            % Support name-value pair arguments when constructing the object.
            setProperties(obj,nargin,varargin{:});
        end
    end
    
    methods (Access=protected)
        function setupImpl(obj) %#ok<MANU>
            if coder.target('Rtw')
                coder.cinclude('colorSensor.h');
                coder.ceval('colorSensor_Init');
            else
                % Place simulation setup code here
            end
        end
        
        function [red,green,blue] = stepImpl(obj)   %#ok<MANU>
            red=double(0);
            green =double(0);
            blue=double(0);
            if coder.target('Rtw')
                % Call C-function implementing device output
                coder.cinclude('colorSensor.h');
                coder.ceval('colorSensor_Step',coder.wref(red),coder.wref(green),coder.wref(blue));
            else
                % Place simulation output code here
                red=0;
                green=0;
                blue=0;
            end
        end
        
        function releaseImpl(obj) %#ok<MANU>
            if coder.target('Rtw')
                % Call C-function implementing device termination
                % No termination code for Arduino
            else
                % Place simulation termination code here
            end
        end
    end
    
    methods (Access=protected)
        %% Define output properties
        function num = getNumInputsImpl(~)
        num = 0; % keine Eingaenge
        end
    
        function num = getNumOutputsImpl(~)
        num = 3; % drei Ausgaenge, rot, gruen, und blau
        end
    
        function varargout = isOutputFixedSizeImpl(~,~)
        varargout{1} = true;
        varargout{2} = true;
        varargout{3} = true;
        end
    
        function varargout = isOutputComplexImpl(~)
        varargout{1} = false;
        varargout{2} = false;
        varargout{3} = false;
        end
    
        function varargout = getOutputSizeImpl(~)
        varargout{1} = [1,1];
        varargout{2} = [1,1];
        varargout{3} = [1,1];
        end
    
        function varargout = getOutputDataTypeImpl(~)
        varargout{1} = 'double';
        varargout{2} = 'double';
        varargout{3} = 'double';
        end
    
        function icon = getIconImpl(~)
        % Define a string as the icon for the System block in Simulink.
        icon = 'TCS34725 color sensor';
        end 
    end
    
    methods (Static, Access=protected)
        function simMode = getSimulateUsingImpl(~)
            simMode = 'Interpreted execution';
        end
        
        function isVisible = showSimulateUsingImpl
            isVisible = false;
        end
    end
    
    methods (Static)
        function name = getDescriptiveName()
            name = 'TCS34725 color sensor';
        end
        
        function b = isSupportedContext(context)
            b = context.isCodeGenTarget('rtw');
        end
        
        function updateBuildInfo(buildInfo, context)
            if context.isCodeGenTarget('rtw')
                % Update buildInfo
                srcDir = fullfile(fileparts(mfilename('fullpath')),'src');
                includeDir = fullfile(fileparts(mfilename('fullpath')),'include');
                libDir =  fullfile(fileparts(mfilename('fullpath')),'libraries');
    
                % Include header files
                addIncludePaths(buildInfo,includeDir);
                addIncludePaths(buildInfo,libDir);
    
                % Include source files
                addSourceFiles(buildInfo,'colorSensor.cpp',srcDir);
                addSourceFiles(buildInfo,'Adafruit_TCS34725.cpp',libDir);
                addSourceFiles(buildInfo,'Adafruit_I2CDevice.cpp',libDir);
    
                boardInfo = arduino.supportpkg.getBoardInfo;
        
                switch boardInfo.Architecture
                    case 'avr'
                        % Add SPI Library - For AVR Based
                        ideRootPath = arduino.supportpkg.getAVRRoot;
                        addIncludePaths(buildInfo, fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'SPI', 'src'));
                        srcFilePath = fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'SPI', 'src');
                        fileNameToAdd = {'SPI.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                        % Add Wire / I2C Library - For AVR Based
                        addIncludePaths(buildInfo, fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'Wire', 'src'));
                        addIncludePaths(buildInfo, fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'Wire', 'src', 'utility'));
                        srcFilePath = fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'Wire', 'src');
                        fileNameToAdd = {'Wire.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                        srcFilePath = fullfile(ideRootPath, 'hardware', 'arduino', 'avr', 'libraries', 'Wire', 'src', 'utility');
                        fileNameToAdd = {'twi.c'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                    case 'sam'
                        % Add SPI Library - For SAM Based
                        libSAMPath = arduino.supportpkg.getSAMLibraryRoot;
                        addIncludePaths(buildInfo, fullfile(libSAMPath, 'SPI','src'));
                        srcFilePath = fullfile(libSAMPath, 'SPI','src');
                        fileNameToAdd = {'SPI.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                        % Add Wire / I2C Library - For SAM Based
                        addIncludePaths(buildInfo, fullfile(libSAMPath, 'Wire', 'src'));
                        srcFilePath= fullfile(libSAMPath, 'Wire', 'src');
                        fileNameToAdd = {'Wire.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                    case 'samd'
                        % Add SPI Library - For SAMD Based
                        libSAMDPath = arduino.supportpkg.getSAMDLibraryRoot;
                        addIncludePaths(buildInfo, fullfile(libSAMDPath, 'SPI'));
                        srcFilePath = fullfile(libSAMDPath, 'SPI');
                        fileNameToAdd = {'SPI.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                        % Add Wire / I2C Library - For SAMD Based
                        addIncludePaths(buildInfo, fullfile(libSAMDPath, 'Wire'));
                        srcFilePath= fullfile(libSAMDPath, 'Wire');
                        fileNameToAdd = {'Wire.cpp'};
                        addSourceFiles(buildInfo, fileNameToAdd, srcFilePath);
                
                    otherwise
                        warning('Unexpected board type. Check again.')
                end
            end
        end
    end
end
