%**************************************************************************
%                   Hochschule Hamm-Lippstadt                             *
%**************************************************************************
% Modul           : neopixel.m                                            *
%                                                                         *
% Datum           : 06. Januar 2023                                       *
%                                                                         *
% Funktion        : MATLAB System Class: Ansteuerung von Neopixel LEDs    *
%                                                                         *
% Implementierung : MATLAB R2022b                                         *
%                                                                         *
% Req. Toolbox    : MATLAB Coder, Simulink Support package for Arduino HW *
%                                                                         *
% Autor           : Christabelle Feunang, Christian Schwinne              *
%                                                                         *
% Bemerkung       : Basiert auf der MATLAB generierten Vorlage "Sink.m"   *
%                                                                         *
%**************************************************************************

classdef neopixel < matlab.System & coder.ExternalDependency
    %
    % System object template for a sink block.
    % 
    % This template includes most, but not all, possible properties,
    % attributes, and methods that you can implement for a System object in
    % Simulink.
    %
    % NOTE: When renaming the class name Sink, the file name and
    % constructor name must be updated to use the class name.
    %
    
    % Copyright 2016-2018 The MathWorks, Inc.
    %#codegen
    %#ok<*EMCA>
    
    properties
        % Public, tunable properties.
    end
    
    properties (Nontunable)
        % Public, non-tunable properties.
    end
    
    properties (Access = private)
        % Pre-computed constants.
    end
    
    methods
        % Constructor
        function obj = neopixel(varargin)
            % Support name-value pair arguments when constructing the object.
            setProperties(obj,nargin,varargin{:});
        end
    end
    
    methods (Access=protected)
        function setupImpl(obj) %#ok<MANU>
            if coder.target('Rtw')
                coder.cinclude('neopixel.h'); % Einbinden des hardwarespezifischen C-Codes
                coder.ceval('neopixel_Init'); % Aufruf der Initialisierungsfunktion in C
            else
                % Place simulation setup code here
            end
        end
        
        function stepImpl(obj,red,green,blue)  %#ok<INUSD>
            if coder.target('Rtw')
                % Call C-function implementing device output
                coder.cinclude('neopixel.h');
                % C-Funktion "neopixel_Step(*r,*g,*b)" mit RGB-Farbwerten als Nur-Lesen-Referenz aufrufen
                coder.ceval('neopixel_Step',coder.rref(red),coder.rref(green),coder.rref(blue));
            end
        end
        
        function releaseImpl(obj) %#ok<MANU>
            if isempty(coder.target)
                % Place simulation termination code here
            else
                % Call C-function implementing device termination
                % In diesem Fall wird keine spezifische Terminationsfunktion benoetigt
            end
        end
    end
    
    methods (Access=protected)
        %% Define input properties
        function num = getNumInputsImpl(~)
            num = 3; % drei Eingaenge, rot, gruen, und blau
        end
        
        function num = getNumOutputsImpl(~)
            num = 0; % keine Ausgaenge
        end
        
        function flag = isInputSizeMutableImpl(~,~)
            flag = false;
        end
        
        function flag = isInputComplexityMutableImpl(~,~)
            flag = false;
        end
        
        function validateInputsImpl(~, u)
            if isempty(coder.target)
                % Sicherstellen, dass Eingangssignale vom Typ uint8_t
                % (byte,Ganzzahlen von 0-255) sind
                validateattributes(u,{'uint8'},{'scalar'},'','u');
            end
        end
        
        function icon = getIconImpl(~)
            % Angezeigter Name des Blocks in Simulink
            icon = 'Neopixel LEDs';
        end
    end
    
    methods (Static, Access=protected)
        function simMode = getSimulateUsingImpl(~)
            simMode = 'Interpreted execution';
        end
        
        function isVisible = showSimulateUsingImpl
            isVisible = false;
        end
    end
    
    methods (Static)
        function name = getDescriptiveName()
            name = 'Neopixel LEDs';
        end
        
        function b = isSupportedContext(context)
            b = context.isCodeGenTarget('rtw');
        end
        
        function updateBuildInfo(buildInfo, context)
            if context.isCodeGenTarget('rtw')
                % Update buildInfo
                srcDir = fullfile(fileparts(mfilename('fullpath')),'src');
                includeDir = fullfile(fileparts(mfilename('fullpath')),'include');
                libDir =  fullfile(fileparts(mfilename('fullpath')),'libraries');
    
                % Include header files
                addIncludePaths(buildInfo,includeDir);
                addIncludePaths(buildInfo,libDir);
    
                % Include source files
                % Hardware-spezifischen C-Code einbinden
                addSourceFiles(buildInfo,'neopixel.cpp',srcDir);
                % NeoPixel-Bibliothek einbinden
                addSourceFiles(buildInfo,'Adafruit_NeoPixel.cpp',libDir);
            end
        end
    end
end
